import pandas as pd
from scipy.stats import entropy

from CalculatingFeatures.helper_functions import convert3DEmpaticaToArray, convertInputInto2d, accelerometerFeatureNames, frequencyFeatureNames
from CalculatingFeatures.calculate_features import calculateFeatures

import sys

def getSampleRate(data):
    try:
        timestamps_diff = data['timestamp'].diff().dropna().mean()
    except:
        raise Exception("Error occured while trying to get the mean sample rate from the data.")
    
    return 1000/timestamps_diff

def extractAccFeaturesFromIntradayData(acc_intraday_data, features, time_segment, filter_data_by_segment):
    acc_intraday_features = pd.DataFrame(columns=["local_segment"] + features)

    if not acc_intraday_data.empty:   
        sample_rate = getSampleRate(acc_intraday_data)
     
        acc_intraday_data = filter_data_by_segment(acc_intraday_data, time_segment)

        if not acc_intraday_data.empty:

            acc_intraday_features = pd.DataFrame()

            # apply methods from calculate features module 
            acc_intraday_features = \
                acc_intraday_data.groupby('local_segment').apply(lambda x: calculateFeatures( \
                convertInputInto2d(x['double_values_0'], x.shape[0]), \
                convertInputInto2d(x['double_values_1'], x.shape[0]), \
                convertInputInto2d(x['double_values_2'], x.shape[0]), \
                fs=int(sample_rate), featureNames=features)) 

            acc_intraday_features.reset_index(inplace=True)

    return acc_intraday_features



def cf_features(sensor_data_files, time_segment, provider, filter_data_by_segment, *args, **kwargs):
    acc_intraday_data = pd.read_csv(sensor_data_files["sensor_data"])

    requested_intraday_features = provider["FEATURES"]
    # name of the features this function can compute
    base_intraday_features_names = accelerometerFeatureNames + frequencyFeatureNames
    # the subset of requested features this function can compute
    intraday_features_to_compute = list(set(requested_intraday_features) & set(base_intraday_features_names))

    # extract features from intraday data
    acc_intraday_features = extractAccFeaturesFromIntradayData(acc_intraday_data,
                                                                    intraday_features_to_compute, time_segment,
                                                                    filter_data_by_segment)

    return acc_intraday_features