import pandas as pd
from scipy.stats import entropy

from CalculatingFeatures.helper_functions import convertInputInto2d, gsrFeatureNames
from CalculatingFeatures.calculate_features import calculateFeatures


def getSampleRate(data):
    try:
        timestamps_diff = data['timestamp'].diff().dropna().mean()
    except:
        raise Exception("Error occured while trying to get the mean sample rate from the data.")

    return 1000/timestamps_diff

def extractEDAFeaturesFromIntradayData(eda_intraday_data, features, time_segment, filter_data_by_segment):
    eda_intraday_features = pd.DataFrame(columns=["local_segment"] + features)

    if not eda_intraday_data.empty:   
        sample_rate = getSampleRate(eda_intraday_data)   
     
        eda_intraday_data = filter_data_by_segment(eda_intraday_data, time_segment)

        if not eda_intraday_data.empty:

            eda_intraday_features = pd.DataFrame()

            # apply methods from calculate features module 
            eda_intraday_features = \
                eda_intraday_data.groupby('local_segment').apply(\
                lambda x: calculateFeatures(convertInputInto2d(x['electrodermal_activity'], x.shape[0]), fs=int(sample_rate), featureNames=features)) 

            eda_intraday_features.reset_index(inplace=True)

    return eda_intraday_features


def cf_features(sensor_data_files, time_segment, provider, filter_data_by_segment, *args, **kwargs):
    eda_intraday_data = pd.read_csv(sensor_data_files["sensor_data"])

    requested_intraday_features = provider["FEATURES"]
    # name of the features this function can compute
    base_intraday_features_names = gsrFeatureNames
    # the subset of requested features this function can compute
    intraday_features_to_compute = list(set(requested_intraday_features) & set(base_intraday_features_names))

    # extract features from intraday data
    eda_intraday_features = extractEDAFeaturesFromIntradayData(eda_intraday_data,
                                                                    intraday_features_to_compute, time_segment,
                                                                    filter_data_by_segment)

    return eda_intraday_features